% Preprocessing of the of matrix M : each column is preprocessed as 
% described in preprocNMFonecol.m. 
%
% See Nicolas Gillis, "Sparse and Unique Nonnegative Matrix Factorization 
% Through Data Preprocessing", arXiv. 
%
%
% *** Input ***
%   M              : (m x n) nonnegative data matrix.
%   epsilon >= 0   : "relxation" allowed on the constraint MB <= M. 
%   solver         : = 1, lsqlin (default) 
%                    = 2, quadprog active set
%                    = 3, quadprog interior point
%   
% *** Output ***  
%   Mp        : preprocessed version of matrix M
%   D         : Inverse positive matrix such that Mp = M*D 
%               rho(I-D) should be strictly smaller than 1 for the rank of
%               Mp to be guaranteed to be equal to the rank of M. 

function [Mp,D] = preprocNMF(M,epsilon,solver);

fprintf('Preprocessing started...\n');

if nargin <= 1, epsilon = 0; end
if nargin <= 2, solver = 1; end

if min(M(:)) < 0
    disp('The matrix contains negative entries, there are set to zero');
    M = max(M,0);
end

[m,n] = size(M);

D = eye(n); 
for i = 1 : n
    [xi,mi] = preprocNMFonecol(M,i,epsilon,solver); 
    Mp(:,i) = mi; 
    D([1:i-1 i+1:n],i) = -xi; 
    if i == 1, fprintf('column %1.0f...',i); 
    elseif mod(i,10) == 0, fprintf('%1.0f...\n',i); 
    else fprintf('%1.0f...',i); end
    if i == n, fprintf('Done. \n',i); end
end

% Check whether rho(B*) < 1 : 
B = eye(n) - D; s = max(abs(eig(B))); 
if s >= 1
    fprintf('spectral radius larger than one, rho(I-D) = rho(B*) = %1.4f \n',s);
    disp('The preprocessed matrix might have set some columns of M to zero')
    disp('and generate a preprocessed matrix Mp s.t. rank(Mp)<rank(M).')
else
    fprintf('spectral radius smaller than one, rho(I-D) = rho(B*) = %1.4f . \n',s);
end