% Example of the preprocessing on the Hubble Telescope hyperspectal image. 

disp('Illustration of the preprocessing on the Hubble dataset.')
disp('It should take a few minuts.')

% load Hubble dataset 
load Hubble; 
% Select a subset of the columns for this example to run faster.
% (You can use the full dataset but it will take more time to process.)
M = M(:,1:4:end); 
[m,n] = size(M); nM = norm(M,'fro'); 


%***** Preprocessing *****
% Preprocessing of the Hubble images using quadprog with interior point
% with epsilon = 0.01
[Mp,Q] = preprocNMF(M,0.01,3); 

% Rescaling of the columns of Mp
dM = sqrt(sum(M.^2)); dMp = sqrt(sum(Mp.^2));
D = diag(dMp.^(-1))*diag(dM);
Mp = Mp*D; Q = Q*D; 

%***** Computing NMF solutions *****
% Factorization rank and maximum number iterations
r = 8; maxiter = 1000; 
% Random initialization
U0 = rand(m,r); V0 = rand(r,n); 

% SVD 
fprintf('Computing truncated SVD...')
[Upca,s,Vpca] = svds(M,r); errorPCA = norm(M-Upca*s*Vpca','fro');
sparuPCA = sparsi(Upca); sparvPCA = sparsi(Vpca);
disp(' done.')

% Standard NMF  (using A-HALS)
fprintf('Computing NMF...')
[Unmf,Vnmf] = HALSacc(M,U0,V0,0.5,0.1,maxiter,+Inf); 
errorNMF = 100*norm(M-Unmf*Vnmf,'fro')/nM;
% Optimizing fixing the sparsity pattern of U 
[Unmfi,Vnmfi] = HALSaccFixed(M,Unmf,Vnmf,0.5,0.1,100,+Inf); 
errorNMFi = 100*norm(M-Unmfi*Vnmfi,'fro')/nM;
sparuNMF = sparsi(Unmfi); sparvNMF = sparsi(Vnmfi);
disp(' done.')

% Standard NMF applied on the preprocessed matrix (using A-HALS)
fprintf('Computing Pre-NMF...')
[Un,Vn] = HALSacc(Mp,U0,V0,0.5,0.1,maxiter,+Inf); 
errorPRENMFq = 100*norm(M-Un*Vn*inv(Q),'fro')/nM; 
% Postprocessing to approximate the original matrix M
Vnp = nnlsHALSupdt(M,Un);  errorPRENMF = 100*norm(M-Un*Vnp,'fro')/nM; 
[Unpi,Vnpi] = HALSaccFixed(M,Un,Vnp,0.5,0.1,100,+Inf); 
sparuPRENMF = sparsi(Unpi); sparvPRENMF = sparsi(Vnpi);
errorPRENMFi = 100*norm(M-Unpi*Vnpi,'fro')/nM; 
disp(' done.')

% Sparse NMF using sparsity of the solution Un above (using A-HALS)
fprintf('Computing sparse NMF (with sparsity of Pre-NMF)...')
sU = sum(Un(:)==0)/m/r; 
[Us,Vs] = sHALSacc(M,U0,V0,sU,0.5,0.1,maxiter,+Inf); 
errorSNMF = 100*norm(M-Us*Vs,'fro')/nM; 
% Optimizing fixing the sparsity pattern of U 
[Usi,Vsi] = HALSaccFixed(M,Us,Vs,0.5,0.1,100,+Inf); 
sparuSNMF = sparsi(Usi); sparvSNMF = sparsi(Vsi);
errorSNMFi = 100*norm(M-Usi*Vsi,'fro')/nM; 
disp(' done.')

% Display results
disp('*****  RESULTS  *****') 
disp('            SVD    NMF    PRENMF         SNMF   ')
fprintf('Error       %0.2f   %0.2f   %0.2f (%0.2f)   %0.2f      \n',errorPCA,errorNMF,errorPRENMF,errorPRENMFq,errorSNMF)
fprintf('Improved    %0.2f   %0.2f   %0.2f           %0.2f      \n',errorPCA,errorNMFi,errorPRENMFi,errorSNMFi)
fprintf('Sparsity U  %0.2f  %0.2f  %0.2f          %0.2f       \n',sparuPCA,sparuNMF,sparuPRENMF,sparuSNMF)
fprintf('Sparsity V  %0.2f   %0.2f   %0.2f           %0.2f    \n',sparvPCA,sparvNMF,sparvPRENMF,sparvSNMF)

% Display basis elements
affichage([Unmfi Unpi Usi],8,128,128); 
title('From top to bottom: NMF, Pre-NMF(0.01), and sparse NMF(0.01)'); 